"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataModel = void 0;
const Path_1 = require("./Path");
const Errors_1 = require("./Errors");
/**
 * Holding the data linked to a given schema
 */
class DataModel {
    /**
     * @param schema node to use as schema for this model
     */
    constructor(schema, options) {
        var _a;
        this.schema = schema;
        this.data = schema.default();
        this.listeners = [];
        this.errors = new Errors_1.Errors();
        this.validate();
        this.history = [JSON.stringify(this.data)];
        this.historyIndex = 0;
        this.historyMax = (_a = options === null || options === void 0 ? void 0 : options.historyMax) !== null && _a !== void 0 ? _a : 50;
    }
    /**
     * Adds a listener to notify when the model is invalidated
     * @param listener the listener to be notified
     */
    addListener(listener) {
        this.listeners.push(listener);
    }
    /**
     * Removes a listener from this model
     * @param listener the listener to be removed
     */
    removeListener(listener) {
        const index = this.listeners.indexOf(listener);
        if (index > -1) {
            this.listeners.splice(index, 1);
        }
    }
    /**
     * Validates the model, updates the history and
     * notifies all listeners that the model is invalidated
     */
    invalidate() {
        this.validate();
        const newHistory = JSON.stringify(this.data);
        if (this.history[this.historyIndex] !== newHistory) {
            this.historyIndex += 1;
            this.history.splice(this.historyIndex, this.historyMax, newHistory);
            if (this.history.length > this.historyMax) {
                this.history.splice(0, 1);
                this.historyIndex -= 1;
            }
        }
        this.silentInvalidate();
    }
    /**
     * Notifies all listeners that the model is invalidated
     */
    silentInvalidate() {
        this.listeners.forEach(l => {
            if (l.invalidated)
                l.invalidated(this);
            if (l.errors)
                l.errors(this.errors);
        });
    }
    /**
     * Resets the full data and notifies listeners
     * @param value new model data
     */
    reset(value) {
        this.data = value;
        this.invalidate();
    }
    /**
     * Gets the data at a specified path
     * @param path path at which to find the data
     * @returns undefined, if the the path does not exist in the data
     */
    get(path) {
        let node = this.data;
        for (let index of path) {
            if (node === undefined)
                return node;
            node = node[index];
        }
        return node;
    }
    /**
     * Updates the date on a path. Node will be removed when value is undefined
     * @param path path to update
     * @param value new data at the specified path
     */
    set(path, value) {
        console.log('Set', path.toString(), JSON.stringify(value));
        if (path.getArray().length === 0) {
            this.reset(value);
            return;
        }
        let node = this.data;
        for (let index of path.pop()) {
            if (node[index] === undefined) {
                node[index] = {};
            }
            node = node[index];
        }
        if (value === undefined || (typeof value === 'number' && isNaN(value))) {
            if (typeof path.last() === 'number') {
                node.splice(path.last(), 1);
            }
            else {
                delete node[path.last()];
            }
        }
        else {
            node[path.last()] = value;
        }
        this.invalidate();
    }
    /**
     * Go one step back in history
     */
    undo() {
        if (this.historyIndex > 0) {
            this.historyIndex -= 1;
            this.data = JSON.parse(this.history[this.historyIndex]);
            this.validate();
            this.silentInvalidate();
        }
    }
    /**
     * Go one step forward in history.
     */
    redo() {
        if (this.historyIndex < this.history.length - 1) {
            this.historyIndex += 1;
            this.data = JSON.parse(this.history[this.historyIndex]);
            this.validate();
            this.silentInvalidate();
        }
    }
    /**
     * Uses the schema to check whether the data is valid
     */
    validate() {
        const path = new Path_1.Path().withModel(this);
        this.errors.clear();
        this.data = this.schema.validate(path, this.data, this.errors);
    }
    error(path, error, ...params) {
        const tempErrors = new Errors_1.Errors();
        tempErrors.add(path, error, params);
        this.listeners.forEach(l => {
            if (l.errors)
                l.errors(tempErrors);
        });
    }
}
exports.DataModel = DataModel;
