"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Path = void 0;
const Registries_1 = require("../Registries");
/**
 * Immutable helper class to represent a path in data
 * @implements {Iterable<PathElement>}
 */
class Path {
    /**
     * @param arr Initial array of path elements. Empty if not given
     * @param model Model attached to this path
     */
    constructor(arr, model) {
        /**
         * Gets the locale of a key from the locale registry.
         *
         * @param key string or path that refers to a locale ID.
         *    If a string is given, an exact match is required.
         *    If a path is given, it finds the longest match at the end.
         * @returns undefined if the key isn't found for the selected language
         */
        this.locale = () => {
            var _a;
            let path = this.arr.filter(e => (typeof e === 'string'));
            while (path.length > 0) {
                const locale = Registries_1.LOCALES.getLocale(path.join('.'));
                if (locale !== undefined)
                    return locale;
                path.shift();
            }
            path = this.arr.filter(e => (typeof e === 'string'));
            while (path.length > 0) {
                const locale = Registries_1.LOCALES.get('en')[path.join('.')];
                if (locale !== undefined)
                    return locale;
                path.shift();
            }
            return ((_a = this.last()) !== null && _a !== void 0 ? _a : '').toString().replace(/^minecraft:/, '');
        };
        this.arr = arr !== null && arr !== void 0 ? arr : [];
        this.model = model;
    }
    /**
     * The last element of this path
     */
    last() {
        return this.arr[this.arr.length - 1];
    }
    /**
     * A new path with the last element removed
     */
    pop() {
        return new Path(this.arr.slice(0, -1), this.model);
    }
    /**
     * A new path with an element added at the end
     * @param element element to push at the end of the array
     */
    push(element) {
        return new Path([...this.arr, element], this.model);
    }
    copy() {
        return new Path([...this.arr], this.model);
    }
    getArray() {
        return this.arr;
    }
    /**
     * Attaches a model to this path and all paths created from this
     * @param model
     */
    withModel(model) {
        return new Path([...this.arr], model);
    }
    getModel() {
        return this.model;
    }
    /**
     * Gets the data from the model if it was attached
     * @returns undefined, if no model was attached
     */
    get() {
        var _a;
        return (_a = this.model) === null || _a === void 0 ? void 0 : _a.get(this);
    }
    /**
     * Sets the value to the model if it was attached
     */
    set(value) {
        var _a;
        (_a = this.model) === null || _a === void 0 ? void 0 : _a.set(this, value);
    }
    /**
     * Gets the error inside this path if the model is attached
     * @returns a html attribute containing the error message
     */
    error() {
        var _a, _b;
        const errors = (_b = (_a = this.model) === null || _a === void 0 ? void 0 : _a.errors.get(this, true)) !== null && _b !== void 0 ? _b : [];
        if (errors.length === 0)
            return '';
        return `data-error="${errors[0].error}"`;
    }
    /**
     * Checks path equality
     * @param other path to compare
     */
    equals(other) {
        return other.arr.length === this.arr.length && other.arr.every((v, i) => v === this.arr[i]);
    }
    /**
     * Checks if this path is inside another path
     * @param other parent path where this path should be inside
     */
    inside(other) {
        return other.arr.every((v, i) => v === this.arr[i]);
    }
    toString() {
        return this.arr
            .map(e => (typeof e === 'string') ? `.${e}` : `[${e}]`)
            .join('')
            .replace(/^\./, '');
    }
    *[Symbol.iterator]() {
        for (const e of this.arr) {
            yield e;
        }
    }
}
exports.Path = Path;
